# Calculate the pitch tracking errors:
# 1. Gross Pitch Error (GPE)
# 2. V->UV and UV->V
# 3. Fine Pitch Error (FPE)
# See Rabiner 1976's paper on pitch tracking eval
# Apr 28, 2008
# by Wei Chu

#!/usr/bin/perl -w
$ARGC = @ARGV;
if ($ARGC != 3)
{
    die("Usage:\n $0 <det file list> <gth file list> <mul flag> \nNote that the detection file has to be the pitch value sampled at 100Hz (step size - 10ms)");
}

$f_det_list = shift @ARGV;
$f_gth_list = shift @ARGV;
$mul_flag = shift @ARGV;

open(LDT, "$f_det_list") or die ("!$f_det_list!\n");
@l_det = <LDT>;
close(LDT);

open(LGT, "$f_gth_list") or die ("!$f_gth_list!\n");
@l_gth = <LGT>;
close(LGT);

if(@l_det != @l_gth) {
    die("Length of $f_det_list does not match the ground truth $f_gth_list.\n");
}

#print OUT "$f_det_list\n";
print "GPE\tMFPE\tVFPE (%)\n";
$agpe = 0; $agpe2 =0; 
$avoe = 0; $aune = 0; 
$amfpe =0; $avfpe =0; 
	   
$advgv = 0; $adugv = 0;
$advgu = 0; $adugu = 0;

#$l_det[0] =~ /\/(.*?)\./;
#print "$1\t";   #print configuration
for ($m = 0; $m < @l_det; $m++) {
    chomp $l_det[$m];
    chomp  $l_gth[$m];
    if ($mul_flag == 1) {
        ($gpe, $gpe2, $mfpe, $vfpe, $dvgv, $dugv, $dvgu, $dugu) = CalErrMulti($l_det[$m], $l_gth[$m]);
    }
    else {
        ($gpe, $gpe2, $mfpe, $vfpe, $dvgv, $dugv, $dvgu, $dugu) = CalErr($l_det[$m], $l_gth[$m]);
    }
    $sgpe = $gpe / $dvgv *100;
    $sgpe2 = $gpe2 / ($dvgv + $dugv) * 100;
    $svoe = $dugv / ($dugv + $dvgv) * 100;
    $sune = $dvgu / ($dvgu + $dugu) * 100;
    $smfpe /= $dvgv;
    $svfpe = sqrt($vfpe / $dvgv - $smfpe * $smfpe);
    $stotal = $dugv + $dvgv + $dvgu + $dugu;
    $suve = ($dugv + $dvgu) / $stotal * 100;
    # print "$l_det[$m]\n";
    # printf "%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n", $sgpe, $sgpe2, $svoe, $sune, $suve, $smfpe, $svfpe;
    
    $agpe += $gpe;
    $agpe2 += $gpe2;
    $avoe += $voe;
    $aune += $une;
    $amfpe += $mfpe;
    $avfpe += $vfpe;
    
    $advgv += $dvgv;
    $adugv += $dugv;
    $advgu += $dvgu;
    $adugu += $dugu;
}
$agpe = $agpe / $advgv *100;
$agpe2 = $agpe2 / ($advgv + $adugv) * 100;
$avoe = $adugv / ($adugv + $advgv) * 100;
$aune = $advgu / ($advgu + $adugu) * 100;
$amfpe /= $advgv;
$avfpe = sqrt($avfpe / $advgv - $amfpe * $amfpe);
$atotal = $adugv + $advgv + $advgu + $adugu;
$auve = ($adugv + $advgu) / $atotal * 100;
printf "%.2f\t%.2f\t%.2f\n", $agpe, $amfpe, $avfpe;

sub CalErr
{
    ($f_det, $f_gth) = @_;
    
    #range factor
    $a = 0.2; #for GPE +- 20% 
    open(DET, "$f_det") or die ("!$f_det!\n");
    @a_det = <DET>;
    close(DET);
    
    open(GTH, "$f_gth") or die ("!$f_gth!\n");;
    @a_gth = <GTH>;
    close(GTH);
    
    if(abs(@a_det - @a_gth) > 10) {
        die("Length of $f_det does not match the ground truth $f_gth.\n");
    }
    
    $mfpe = 0;  $vfpe = 0;
    $gpe = 0;   $gpe2 = 0;
    $voe = 0;   $une = 0;
    $dvgv = 0;  $dugv = 0;
    $dvgu = 0;  $dugu = 0;   $total = 0;
    
    for ($i = 0; $i < @a_gth; $i++)
    {
        $g = $a_gth[$i];
        if ($g >= 0.0)
        {
            #if g = -1 or -xx skip this problematic frame
            if ($i < @a_det)
            {
                $d = $a_det[$i];
            }
            else
            {
                #0 padding...
                $d = 0;
            }
            
            if ($g > 0.01 && $d > 0.01)
            {
                $dvgv++;
                if ($d < ($g*(1-$a)) || $d > ($g*(1+$a)))
                {
                    #GPE
                    $gpe++;
                    $gpe2++;
                }
                else
                {
                    #FPE
                    $tmp = $g - $d;
                    $mfpe += $tmp;
                    $vfpe += $tmp * $tmp;
                }
            }
            elsif ($g > 0.01 && $d < 0.01)
            {
                #V -> UV
                $dugv++;
                $gpe2++;
            }
            elsif ($g < 0.01 && $d >0)
            {
                #V -> UV
                $dvgu++;
            }
            else
            {
                #UV - UV
                $dugu++;
            }
        }
        $total++;
    }
  
    @ret = ($gpe, $gpe2, $mfpe, $vfpe, $dvgv, $dugv, $dvgu, $dugu);
    return(@ret);    
}


sub CalErrMulti
{
    ($f_det, $f_gth) = @_;
    
    print "$f_gth\n\n";
    #range factor
    $a = 0.2; #for GPE +- 20% 
    open(DET, "$f_det") or die ("!$f_det!\n");
    @a_det = <DET>;
    chomp @a_det;
    close(DET);
    
    open(GTH, "$f_gth") or die ("!$f_gth!\n");;
    @a_gth = <GTH>;
    chomp @a_gth;
    close(GTH);
    
    if(abs(@a_det - @a_gth) > 10) {
        die("Length of $f_det does not match the ground truth $f_gth.\n");
    }
    
    $mfpe = 0;  $vfpe =0;
    $gpe = 0;   $gpe2 = 0;
    $voe = 0;   $une = 0;
    $dvgv = 0;  $dugv = 0;
    $dvgu = 0;  $dugu =0;   $total = 0;
    
    for ($i = 0; $i < @a_gth; $i++)
    {
        $g = $a_gth[$i];
        if ($g >= 0.0)
        {
            #if g = -1 or -xx skip this problematic frame
            undef @da;
            if ($i < @a_det) {
                @da = split(/\s+/, $a_det[$i]);
            }
            else {
                $da[0] = 0.0;
            }
            
            if ($g > 0.01 && $da[0] > 0.01)
            {
                $dvgv++;
                # search the smallest difference
                $mj = 0;
                for ($j = 0; $j < 5; $j++) {
                    if ($da[$j] > ($g*(1-$a)) && $da[$j] < ($g*(1+$a))) {
                        $mj = $j;
                        last;
                    }
                }
                # print "$j\n";
                if ($j == 5)
                    {
                        #GPE
                        $gpe++;
                        $gpe2++;
                    }
                    else
                    {
                        #FPE
                        $tmp = $g - $mid;
                        $mfpe += $tmp;
                        $vfpe += $tmp * $tmp;
                }
            }
            elsif ($g > 0.01 && $da[0] < 0.01)
            {
                #V -> UV
                $dugv++;
                $gpe2++;
                print "0\n";
            }
            elsif ($g < 0.01 && $da[0] >0)
            {
                #V -> UV
                $dvgu++;
                print "0\n";
            }
            else
            {
                #UV - UV
                $dugu++;
                print "0\n";
            }
        }
        $total++;
    }
  
    @ret = ($gpe, $gpe2, $mfpe, $vfpe, $dvgv, $dugv, $dvgu, $dugu);
    return(@ret);    
}
