////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    io_shell.c:                                                         //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#ifndef IO_SHELL_H
#define IO_SHELL_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <time.h>
#include <ctype.h>
#include <float.h>
#include <stddef.h>
#include <stdbool.h>

#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"

#define SPA_FREE_YES	1
#define SPA_FREE_NO	0

#define SPA_SWAP_YES 1
#define SPA_SWAP_NO  0

#define SPA_MAX_TAG_LEN 100
#define SPA_MAX_STR_LEN 1000
#define MAX_CHAR_NUM_IN_A_LINE (9999) 	// max num of characters in a line

//typedef enum {false=0, true=1} bool;

// Structure for HTK header
typedef struct {
	int nSamples;
	int sampPeriod;
	short sampSize;
	short sampKind;
} spa_htk_header;


typedef struct _spa_wav_form {
	int IDRiff;
	int ckLen;
	int IDWave;
} spa_wav_form;

typedef struct _spa_wav_format_chunk {
	int IDfmt;
	int ckLen;
	short PCM;
	short ChannelNum;
	int SampleRate;
	int TransRate;
	short BytePerSample;
	short BitPerSample;
//FORMAT_CHUNK.ckLen == 0x12 or 0x10
//	short BlockAlign;
} spa_wav_format_chunk;

typedef struct _spa_wav_fact_chunk {
	int IDfact;
	int ckLen;
	int SampleNum;
} spa_wav_fact_chunk;

typedef struct _spa_wav_data_chunk {
	int IDdata;
	int ckLen;
} spa_wav_data_chunk;


//���ļ�ͷ
typedef struct _spa_wav_head{
	char	Riff[4];
	int	FileLength;
	char	Wave[4];
	char	fmt[4];
	int	TempBytes;
	short	PCM;
	short	nChannel;
	int	SampleRate;
	int	TransRate;
	short	Adjust;
	short	nBitsPerSample;
	char	cData[4];
	int	nBytes;
} spa_wav_head;

int spa_io_saybye(char *msg);
void spa_io_lookout(char *msg);

// Get Next Tag from 'fp', tag stored in 'ctag'
// Return Value:
//      0 : Unresolved TAG; 1 : Success; -1 : End of file
int spa_io_get_next_tag(FILE *fp, char *ctag);

// open file, tag: "rt", "wt+" ...
FILE *spa_io_fopen(char *fn, char *tag);
void spa_io_fclose(FILE *fp);

// lines in a text file
int spa_io_count_lines_of_textfile(char *file);

// Read and write wav files
int spa_io_read_htk_header(char *fn, int swap, spa_htk_header **ph);
int spa_io_read_htk_mat(char *fn, int swap, gsl_matrix **pm);
int spa_io_read_htk_buf(char *fn, int swap, double ***pm, int *frm, int *dim);
int spa_io_read_htk_feamat(char *fn, int swap, spa_htk_header **ph, gsl_matrix **pm);
int spa_io_read_htk_feabuf(char *fn, int swap, spa_htk_header **ph, double ***pm, int *frm, int *dim);

int spa_io_write_htk_header(FILE *fp, spa_htk_header *header);
int spa_io_write_htk_vec(FILE *fp, gsl_vector *v);
int spa_io_write_htk_arr(FILE *fp, double *v, int s);
int spa_io_write_htk_feamat(char *fn, spa_htk_header *header, gsl_matrix *m);
int spa_io_write_htk_feabuf(char *fn, spa_htk_header *header, double **m, int row, int col);

void spa_io_release_htk_header(spa_htk_header *h);
void spa_io_release_htk_mat(gsl_matrix *m);
void spa_io_release_htk_buf(double **m);
void spa_io_release_htk_feamat(spa_htk_header *h, gsl_matrix *m);
void spa_io_release_htk_feabuf(spa_htk_header *h, double **m);

// Read and write wave file
// return sampling rate
int spa_io_read_wav(char *fn, gsl_vector **pw, int cfg_smp_rate);
void spa_io_write_wav(char *fn, gsl_vector *w, int smp_rate, int free_flag);

void spa_io_fread_gsl_matrix(FILE *fp, gsl_matrix **pm);
int spa_io_fload_gsl_matrix(char *fn, gsl_matrix **pm);
void spa_io_fwrite_gsl_matrix(FILE *fp, gsl_matrix *m);
int spa_io_fsave_gsl_matrix(char *fn, gsl_matrix *m);
void spa_io_fwrite_gsl_vector(FILE *fp, gsl_vector *v);

void ShowVec(double *v, int s);
void ShowMat(double **m, int r, int l);
// set the elements of vector and matrix to be all 0s.
void spa_io_zero_vec(double *v, int s);
void spa_io_zero_mat(double **m, int r, int l);
void spa_io_zero_cub(double ***m, int x, int y, int z);
void spa_io_zero_qua(double ****m, int x, int y, int z, int w);
void spa_io_copy_int_vec(int *dst, int *src, int s);
void spa_io_copy_vec(double *dst, double *src, int s);
void spa_io_copy_mat(double **dst, const double **src, int r, int l);
void spa_io_copy_cub(double ***dst, double ***src, int x, int y, int z);
void spa_io_copy_qua(double ****dst, double ****src, int x, int y, int z, int w);
void spa_io_zero_int_vec(int *v, int s);
void spa_io_zero_int_mat(int **m, int r, int l);
void spa_io_zero_int_cub(int ***m, int x, int y, int z);
void spa_io_zero_int_qua(int ****m, int x, int y, int z, int w);

// set the elements of vector and matrix to be all d.
void spa_io_init_int_vec(int *v, int s, int a);
void spa_io_init_vec(double *v, int s, double d);
void spa_io_init_mat(double **m, int r, int l, double d);

void spa_io_get_suffix (char *src, char *suffix);
void spa_io_replace_suffix (char *dst, char *src, char *suffix);
void spa_io_insert_suffix (char *dst, char *src, char *insert);

int spa_io_flip_flag(int *ipFlag, int s);
				
char         **alloc2d(int dim1,	// "x" dimension 
		       int dim2,	// "y" dimension
	 int size	// number of bytes each entry takes
		      );
void free2d(char **p);

char      ***alloc3d( int       dim1,        // "x" dimension
		      int 	dim2,        // "y" dimension
	int 	dim3,        // "z" dimension
 int 	size         // number of bytes per entry
		    );
void  free3d(char ***X);

char      ***alloc4d ( int      dim1,       // "x" dimension
		       int 	dim2,        // "y" dimension
	 int 	dim3,        // "z" dimension
  int	dim4,	     // "w" dimension
  int 	size         // number of bytes per entry
		     );
void free4d(char ****X);

#endif // IO_SHELL_H
