////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    pitch.c: F0 related algorithms                                      //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

//#define SPA_PITCHEXT_DEBUG 1
//#define SPA_PITCH_DEBUG 0
#include "pitch.h"

extern char *msg;
// init pitch parameters 
int spa_spec_pch_init(char *fn, spa_spec_param *par, spa_pitch_param** ppch)
{
	char ctag[SPA_MAX_TAG_LEN];	// content in the <...> 
	int i, j, k;
	double d, lobd, upbd, h2s, rstep;
	FILE *fp;

	spa_pitch_param *pch = (spa_pitch_param *) calloc(sizeof(spa_pitch_param), 1);
	pch->frm_len_sec	= par->frm_len_t * 1e-3;	// ms -> s 
	pch->cand_num 		= 100;
	pch->spec_smth_len	= 50;
	pch->f_min		= 50;
	pch->f_max		= 400;
	pch->f_cut		= 3000;
	pch->abs_snr_num	= 6;
	pch->dp_ratio		= 0.08;
	pch->octave_cost	= 0.35;
	pch->resi_space		= 0.005;
	pch->snr_band_resi	= NULL;
	pch->trans_cost_table	= NULL;
	pch->peak_flag		= SPA_PITCH_USE_PROMINENT_PEAK;
	pch->freq_band_num	= 3;
	pch->lr_sca		= 0.33;
	pch->lr_exp		= 1.0;
	
	fp = spa_io_fopen(fn, "rt");	
	while ( spa_io_get_next_tag(fp, ctag) == 1) {
		if ( strcmp(ctag, "BAND_NUM") == 0 ) {
			fscanf(fp, "%d", &pch->freq_band_num);
		}
		else if ( strcmp(ctag, "DIFF_SNR_NUM") == 0 ) {
			fscanf(fp, "%d", &pch->diff_snr_num);
			fscanf(fp, "%lf", &lobd);
			fscanf(fp, "%lf", &upbd);
			pch->diff_snr_upbd = (double *) calloc(pch->diff_snr_num, sizeof(double));
			pch->diff_snr_lobd = (double *) calloc(pch->diff_snr_num, sizeof(double));
			// -max - -1 ... 1 +max 
			pch->diff_snr_lobd[0] = -DBL_MAX;			
			pch->diff_snr_upbd[0] = lobd;
			pch->diff_snr_lobd[1] = lobd;
			for (i = 1; i < pch->diff_snr_num - 2; i++) {
				d = lobd + (upbd - lobd) * i / (double) (pch->diff_snr_num - 2);
				pch->diff_snr_upbd[i] = d;
				pch->diff_snr_lobd[i+1] = d;
			}
			pch->diff_snr_upbd[pch->diff_snr_num - 2] = upbd;
			pch->diff_snr_lobd[pch->diff_snr_num - 1] = upbd;
			pch->diff_snr_upbd[pch->diff_snr_num - 1] = DBL_MAX;
		}
		else if ( strcmp(ctag, "F0_CAND_NUM") == 0 )
			fscanf(fp, "%d", &pch->cand_num);
		else if ( strcmp(ctag, "USE_PROMINENT_PEAK") == 0 )
			fscanf(fp, "%d", &pch->peak_flag);
		else if ( strcmp(ctag, "F0_MIN") == 0 )
			fscanf(fp, "%lf", &pch->f_min);
		else if ( strcmp(ctag, "F0_MAX") == 0 )
			fscanf(fp, "%lf", &pch->f_max);
		else if ( strcmp(ctag, "SPEC_CUT") == 0 )
			fscanf(fp, "%lf", &pch->f_cut);
		else if ( strcmp(ctag, "ABSO_SNR_NUM") == 0 ) {
			// convert dB to linear 
			fscanf(fp, "%d", &pch->abs_snr_num);
			fscanf(fp, "%lf", &lobd);
			fscanf(fp, "%lf", &upbd);
			pch->abs_snr_upbd = (double *) calloc(pch->abs_snr_num, sizeof(double));
			pch->abs_snr_lobd = (double *) calloc(pch->abs_snr_num, sizeof(double));
			pch->abs_snr_lobd[0] = -DBL_MAX;
			for (i = 0; i < pch->abs_snr_num - 1; i++) {
				d = lobd + (upbd - lobd) * i / (double) (pch->abs_snr_num - 2);
				pch->abs_snr_upbd[i] = d;
				pch->abs_snr_lobd[i+1] = d;
			}
			pch->abs_snr_upbd[pch->abs_snr_num - 1] = DBL_MAX;	
		}
		else if ( strcmp(ctag, "DP_RATIO") == 0 )
			fscanf(fp, "%lf", &pch->dp_ratio);
		else if ( strcmp(ctag, "OCTAVE_COST") == 0 )
			fscanf(fp, "%lf", &pch->octave_cost);
		else if ( strcmp(ctag, "LR_SCA") == 0 )
			fscanf(fp, "%lf", &pch->lr_sca);
		else if ( strcmp(ctag, "LR_EXP") == 0 )
			fscanf(fp, "%lf", &pch->lr_exp);
	}
	spa_io_fclose(fp);
	
	h2s = par->fft_num / (double) par->smp_rate;
	pch->hz_per_smp = h2s;
	pch->k_max = pch->f_max * h2s;					// convert Hz to samples 
	pch->k_min = pch->f_min * h2s;
	pch->harm_num = round(pch->f_cut * h2s) + 1;
	
	pch->cands = (double *) calloc(pch->cand_num, sizeof(double));
	pch->harms = (double *) calloc(pch->harm_num, sizeof(double));
	rstep = pow(pch->f_max/pch->f_min, 1.0/(double) (pch->cand_num - 1));
	pch->cands[0] = pch->f_min;
	for (i = 1; i < pch->cand_num - 1; i++) {
		pch->cands[i] = pch->cands[i - 1] * rstep;
	}
	pch->cands[pch->cand_num - 1] = pch->f_max;
	
	for (i = 0; i < pch->harm_num; i++) {
		pch->harms[i] = i / h2s;
	}

	if (pch->freq_band_num > 0) {
		d = pch->f_cut / (double) pch->freq_band_num;
		pch->freq_band_upbd = (double *) calloc(pch->freq_band_num, sizeof(double));
		pch->freq_band_lobd = (double *) calloc(pch->freq_band_num, sizeof(double));
		pch->freq_band_lobd[0] = -1.0;	
		for (i = 0; i < pch->freq_band_num - 1; i++) {
			pch->freq_band_upbd[i] = d * (i+1);
			pch->freq_band_lobd[i+1] = d * (i+1);
		}
		pch->freq_band_upbd[pch->freq_band_num - 1] = pch->f_cut + 1.0;
	}
	
	*ppch = pch;	
	return GSL_SUCCESS;
}

// release pitch parameters 
int spa_spec_pch_release(spa_pitch_param* pch)
{
	int i, j, k, n;
	if (pch->snr_band_resi != NULL) {
		for (k = 0; k < pch->abs_snr_num; k++) {
			for (n = 0; n < pch->freq_band_num; n++) {
				gsl_vector_free(pch->snr_band_resi[k][n].pdf_table);
			}
		}
		free2d((char **)pch->snr_band_resi);
	}
	if (pch->trans_cost_table != NULL) {
		gsl_matrix_free(pch->trans_cost_table);
	}
	
	free(pch->abs_snr_upbd);
	free(pch->abs_snr_lobd);
	free(pch->diff_snr_upbd);
	free(pch->diff_snr_lobd);
	free(pch->freq_band_upbd);
	free(pch->freq_band_lobd);
	free(pch->cands);
	free(pch->harms);
	free(pch);
	return GSL_SUCCESS;
}


// load reference F0 countour 
void spa_load_pch_ref_file(char *pch_file, gsl_vector **ppch_ref)
{
	FILE *fp;
	double d;
	int i, count;
	gsl_vector *pch_ref;
	
	if ((fp = fopen(pch_file, "rt")) == NULL) {
		printf("spa_load_pch_ref_file: Cannot open F0 reference file %s for read.\n", pch_file);
	};
	
	count = 0;
	while(!feof(fp)) {
		if(fscanf(fp, "%lf\n", &d) != 1)
			break;
		count++;
	}
	pch_ref = gsl_vector_calloc(count);
	rewind(fp);
	i = 0;
	while(!feof(fp)) {
		if(fscanf(fp, "%lf\n", &d) != 1)
			break;
		gsl_vector_set(pch_ref, i, d);
		i++;
	}
	
	fclose(fp);
	*ppch_ref = pch_ref;
}


// Search the local peaks in the smoothed SNR spectrum 
// peak info is filled into peak_arr 
void spa_spec_local_peak(gsl_vector *short_spec_smth, gsl_vector *long_spec_smth, spa_pitch_param *pch, spa_vector_spec_peak *peak_arr)
{
	int i, j, st, et, ti, peak_num;
	int *peak_idx;								// the indices of locations of selected peak 
	spa_spec_peak *pv;
	double dt = 0.0, dl1, dl2, dc, dr1, dr2, der, acc, kappa, smth_len, ham_sum, base, prv, cur, snr;	
	gsl_vector *diff_snr;							// for storing the difference of SNR at the peaks 
	
	// find the num of all local peaks 
	peak_num = 0;
	for (i = 2; i < pch->harm_num - 2; i++) {
		if ( i > pch->k_min) {
			dl2 = gsl_vector_get(short_spec_smth, i-2);
			dl1 = gsl_vector_get(short_spec_smth, i-1);
			dc  = gsl_vector_get(short_spec_smth, i);
			dr1 = gsl_vector_get(short_spec_smth, i+1);
			dr2 = gsl_vector_get(short_spec_smth, i+2);
			if ( dc > dl1 && dc > dl2 && dc > dr1 && dc > dr2 ) {
				peak_num++;
			}
		}
	}
	
	// fill in peak information into  
	peak_idx = (int *)calloc(peak_num, sizeof(int));
	peak_arr->pk_vec = (spa_spec_peak *) calloc(peak_num, sizeof(spa_spec_peak));
	pv = peak_arr->pk_vec;
	peak_arr->pk_num = peak_num;
	diff_snr = gsl_vector_calloc(peak_num);
	j = 0;
	for (i = 2; i < pch->harm_num - 2; i++) {
		if ( i > pch->k_min) {
			dl2 = gsl_vector_get(short_spec_smth, i-2);
			dl1 = gsl_vector_get(short_spec_smth, i-1);
			dc  = gsl_vector_get(short_spec_smth, i);
			dr1 = gsl_vector_get(short_spec_smth, i+1);
			dr2 = gsl_vector_get(short_spec_smth, i+2);
			if ( dc > dl1 && dc > dl2 && dc > dr1 && dc > dr2 ) {
				// the long-term signification of the peak;
				pv[j].hid = i;
				pv[j].harm = pch->harms[i];
				pv[j].abs_sid = spa_search_match_bin(pch->abs_snr_lobd, pch->abs_snr_upbd, pch->abs_snr_num, dc);
				pv[j].abs_snr = dc;
				pv[j].bid = spa_search_match_bin(pch->freq_band_lobd, pch->freq_band_upbd, pch->freq_band_num, pv[j].harm);
				gsl_vector_set(diff_snr, j, dc - gsl_vector_get(long_spec_smth, i));
				j++;
			}
		}
	}
	if (j != peak_num) {
		printf("spa_spec_local_peak: peak numbers mismatch.\n");
	}
	
	// normalization of diff_snr 
	spa_math_norm_vector(diff_snr);
	for (j = 0; j < peak_num; j++) {
		dc = gsl_vector_get(diff_snr, j);
		pv[j].diff_snr = dc;
		pv[j].diff_sid = spa_search_match_bin(pch->diff_snr_lobd, pch->diff_snr_upbd, pch->diff_snr_num, dc);
	}
	
#ifdef SPA_PITCH_DEBUG
	FILE *gfp = fopen("debug.txt", "wt");
	for (j = 0; j < peak_num; j++) {
		fprintf(gfp, "%f\n", pv[j].harm);
	}
	fclose(gfp);
	gfp = fopen("debug.txt", "wt");
	for (j = 0; j < peak_num; j++) {
		fprintf(gfp, "%f\n", pv[j].diff_snr);
	}
	fclose(gfp);
#endif
	
	// free 	
	free(peak_idx);
	gsl_vector_free(diff_snr);
}

// extract pitch candidates or learn pitch model from the wav*/
// uv_ref: for training, F0 ground truth values; for testing, storing the Unvoice/Voiced boundaries 
// the peak information is filled into the ppeak_mat 
void spa_pitch_peak_cand (spa_spec_param *par, 
			  spa_pitch_param *pch, 
			  gsl_vector *sph,
			  gsl_vector *uv_ref,
			  spa_vector_spec_peak **ppeak_mat,
			  int *pfrm_num)
{
	int i, j, k, m, in, frm_num, st, st_org, smp_num, cand_num, harm_num, noise_num, peak_num;
	double raw_eng, d, sum, pch_stage, f, *freqs, res;
	double k_max, k_min, k_cut, k_res_1st, h2s;
	char *pch_file, *cand_file;
	spa_vector_spec_peak *peak_mat;
	gsl_vector *sph_frm, *spec_amp, *enos_amp, *spec_snr, *bp_filt;
	gsl_matrix *cand_pp;								// post prob of F0 candidates in this frame 
	//spa_pitch_cand_pmf *cand_arr;							// F0 candidates location and likelihood 
	gsl_vector *cand_pmf;
	FILE *fp_mod, *fp_cand;
	
	// smooth the spectrum : length of the smoothing window : 2 x len + 1 
	int short_smth_len = floor(0.5 * (pch->k_min - 1));				// short term smoothing 
	int long_smth_len = floor(0.5 * (pch->k_max - 1));				// long term spectrum 
	gsl_vector *log_spec_snr = gsl_vector_calloc(pch->harm_num);
	gsl_vector *short_spec_smth = gsl_vector_calloc(pch->harm_num);
	gsl_vector *long_spec_smth = gsl_vector_calloc(pch->harm_num);
	gsl_vector *short_ham_win = spa_spec_hamming_win(2*short_smth_len + 1);
	gsl_vector *long_ham_win = spa_spec_hamming_win(2*long_smth_len + 1);	
	
	// strict timing: center on the (i-0.5)*d), duration: [(i-0.5)*d-L/2 (i-1)*d+L/2-1] 
	frm_num = (int) sph->size/par->frm_step + 0.5;
	st_org = (int) (0.5* (par->frm_step - par->frm_len));
	cand_pp = gsl_matrix_calloc(frm_num, pch->cand_num);
	// alloc mem for storing the peaks in each frame 
	peak_mat = (spa_vector_spec_peak *)calloc(frm_num, sizeof(spa_vector_spec_peak));
	*ppeak_mat = peak_mat;
	*pfrm_num = frm_num;
	
	sph_frm		= gsl_vector_calloc(par->frm_len);	// keep a frame of speech 
	spec_amp	= gsl_vector_calloc(par->fft_half_num); // spectrum of the framed speech 
	enos_amp	= gsl_vector_calloc(par->fft_half_num); // estimated spectrum of the noise 
	spec_snr	= gsl_vector_calloc(pch->harm_num);	// SNR across frequency bins (only 0 - f_cut hz)
	
	// caculate the SNR level (in log) 
	spa_estimate_noise_spectrum (sph, sph_frm, frm_num, st_org, par, enos_amp);
	
#ifdef SPA_PITCH_DEBUG
	FILE *gfp = fopen("debug.txt", "wt");
	gsl_vector_fprintf(gfp, enos_amp, "%e");
	fclose(gfp);
#endif
	//printf("Processed:\n");
	// caculate candidates for each voiced frame 
	for (i = 0; i < frm_num; i++) {
		//printf("%d\/%d\n", i, frm_num);
		if (i >= uv_ref->size || gsl_vector_get(uv_ref, i) < 0.5) {
			continue;
		}
		st = st_org + i * par->frm_step;
		spa_spec_get_frm(sph, sph_frm, st);
		spa_get_frm_spec(sph_frm, par, spec_amp, SPA_LINEAR_SPEC);
		
		// spec_amp / enos_amp => spec_snr 
		for (j = 0; j < pch->harm_num; j++) {
			d = gsl_vector_get(spec_amp, j) / gsl_vector_get(enos_amp, j);
			gsl_vector_set(spec_snr, j, d);
		}
		
#ifdef SPA_PITCH_DEBUG
		FILE *gfp = fopen("debug.txt", "wt");
		gsl_vector_fprintf(gfp, spec_amp, "%e");
		fclose(gfp);
#endif
				
#ifdef SPA_PITCH_DEBUG
		gfp = fopen("debug.txt", "wt");
		gsl_vector_fprintf(gfp, sph_frm, "%e");
		fclose(gfp);
#endif
		

		// smooth the spectrum 
		gsl_vector_memcpy(log_spec_snr, spec_snr);
		spa_spec_vector_linear2log(log_spec_snr);
		spa_spec_vector_log2db(log_spec_snr, par->use_pow);

#ifdef SPA_PITCH_DEBUG
		gfp = fopen("debug.txt", "wt");
		gsl_vector_fprintf(gfp, log_spec_snr, "%e");
		fclose(gfp);
#endif
		
		spa_math_smoothing(log_spec_snr, short_ham_win, short_spec_smth);
		spa_math_smoothing(log_spec_snr, long_ham_win, long_spec_smth);
	
#ifdef SPA_PITCH_DEBUG
		gfp = fopen("debug.txt", "wt");
		gsl_vector_fprintf(gfp, short_spec_smth, "%e");
		fclose(gfp);
		
		gfp = fopen("debug.txt", "wt");
		gsl_vector_fprintf(gfp, long_spec_smth, "%e");
		fclose(gfp);
#endif
		// get peaks information of SNR spectrum 
		spa_spec_local_peak(short_spec_smth, long_spec_smth, pch, &peak_mat[i]);
	}
	
	free(cand_pp);
	gsl_vector_free(sph_frm);
	gsl_vector_free(spec_amp);
	gsl_vector_free(enos_amp);
	gsl_vector_free(spec_snr);
	gsl_vector_free(log_spec_snr);
	gsl_vector_free(short_spec_smth);
	gsl_vector_free(long_spec_smth);
	gsl_vector_free(short_ham_win);
	gsl_vector_free(long_ham_win);
}

// accumulate the occurence of F0 candidates
// spa_pitch_param *pch : 		input
// spa_vector_spec_peak *peak_mat :	input, 
// int frm_num :			input
// gsl_vector *pch_ref:			input
// spa_clus_resi_info **psb_resi:	output
void spa_pitch_accu_cand_occurrence(spa_pitch_param *pch, 
				    spa_vector_spec_peak *peak_mat, 
				    int frm_num,
				    gsl_vector *pch_ref,
				    spa_clus_resi_info ***psb_resi)
{
	int i, j, k, multi, pn;
	double ratio, resi, fv, d;
	int fid, sid, bid, ***count;
	spa_spec_peak *pv;
	spa_resi_info *rv;
	
	for (fid = 0; fid < pch->cand_num; fid++) {
		for (sid = 0; sid < pch->abs_snr_num; sid++) {
			for (bid = 0; bid < pch->freq_band_num; bid++) {
				psb_resi[fid][sid][bid].resi_num = 0;
			}
		}
	}
	for (i = 0; i < frm_num; i++) {
		if (peak_mat[i].pk_num == 0 || i >= pch_ref->size) {
			// an unvoiced frame 
			continue;
		}
		fv = gsl_vector_get(pch_ref, i);
		fid = round ((log(fv)/ log(pch->f_min) - 1) * pch->cand_num);
		// prevent out of range 
		if (fid < 0) {
			fid = 0;
		}
		else if (fid >= pch->cand_num) {
			fid = pch->cand_num - 1;
		}
		for (j = 0; j < peak_mat[i].pk_num; j++) {
			// find the residual for the peaks has highest relative SNR (> 0.33)
			pv = &peak_mat[i].pk_vec[j];
			
			// here is a switch of accumulating prominent peak or non-prominent peak*/
			if ( (pv->diff_sid != pch->diff_snr_num - 1 && pch->peak_flag == SPA_PITCH_USE_PROMINENT_PEAK) || // use prominent peak, discard non prominent peak
				(pv->diff_sid == pch->diff_snr_num - 1 && pch->peak_flag == SPA_PITCH_USE_NON_PROMINENT_PEAK) // use non-prominent peak, discard prominent peak
			   ){
				pv->resi = -1.0;	// an impossible value 
				pv->fid = 0;
			}
			else {
				// otherwise regard it as a peak 
				ratio = pv->harm / fv;
				pv->multi = round(ratio);
				pv->resi = ratio - pv->multi;
				pv->pch = fv;
				pv->fid = fid;
				psb_resi[fid][pv->abs_sid][pv->bid].resi_num++;
			}
		}
	}
	
	// alloc mem for storing statistics of residuals 
	for (fid = 0; fid < pch->cand_num; fid++) {
		for (sid = 0; sid < pch->abs_snr_num; sid++) {
			for (bid = 0; bid < pch->freq_band_num; bid++) {
				if (psb_resi[fid][sid][bid].resi_num == 0) {
					psb_resi[fid][sid][bid].resi_vec = NULL;
				}
				else {
					psb_resi[fid][sid][bid].resi_vec = (spa_resi_info *) calloc(psb_resi[fid][sid][bid].resi_num, sizeof(spa_resi_info));
				}
			}
		}
	}
	
	count = (int ***) alloc3d(pch->cand_num, pch->abs_snr_num, pch->freq_band_num, sizeof(int));
	spa_io_zero_int_cub(count, pch->cand_num, pch->abs_snr_num, pch->freq_band_num);
	for (i = 0; i < frm_num; i++) {
		for (j = 0; j < peak_mat[i].pk_num; j++) {
			// find the residual for the peaks having highest relative SNR (> 0.33)
			pv = &peak_mat[i].pk_vec[j];
			fid = pv->fid;
			sid = pv->abs_sid;
			bid = pv->bid;
			if (fid != 0) { // prominent peak 
				// residual info 
				k = count[fid][sid][bid];
				psb_resi[fid][sid][bid].resi_vec[k].multi	= pv->multi;
				psb_resi[fid][sid][bid].resi_vec[k].resi	= pv->resi;
				psb_resi[fid][sid][bid].resi_vec[k].abs_snr		= pv->abs_snr;
				count[fid][sid][bid]++;
			}
		}
	}
	
	free3d((char ***)count);	
}

// generate posterior score for each F0 candidates in a frame given spectral peaks 
void spa_pitch_cand_postprob (spa_pitch_param *pch, spa_vector_spec_peak *peak_mat, int frm_num, gsl_matrix **pcand_pp) 
{
	int i, j, k, rid, peak_num, *peak_idx;
	double p_resi, p_multi, p_snr, p, q, p_sum, d, fv, ratio, resi, curr_wt, wt_sum;
	gsl_vector *peak_pp, *peak_wt;
	gsl_matrix *cand_pp;
	spa_spec_peak *pv;
	FILE *gfp;
	
	cand_pp = gsl_matrix_calloc(frm_num, pch->cand_num);
	peak_pp = gsl_vector_calloc(pch->cand_num);
	gsl_matrix_set_zero(cand_pp);
	
	for (i = 0; i < frm_num; i++) {
		if (peak_mat[i].pk_num == 0) {
			// an unvoiced frame
			continue;
		}
		
		peak_idx = calloc(peak_mat[i].pk_num, sizeof(int));
		peak_wt = gsl_vector_calloc(peak_mat[i].pk_num);
		wt_sum = 0;
		for (j = 0; j < peak_mat[i].pk_num; j++) {
			pv = &peak_mat[i].pk_vec[j];
			// use prominent peak, discard non prominent peak, or use non-prominent peak, discard prominent peak
			if ( (pv->diff_sid != pch->diff_snr_num - 1 && pch->peak_flag == SPA_PITCH_USE_PROMINENT_PEAK) || 
						   (pv->diff_sid == pch->diff_snr_num - 1 && pch->peak_flag == SPA_PITCH_USE_NON_PROMINENT_PEAK) 
			   ){
				peak_idx[j] = 0;
				gsl_vector_set(peak_wt, j, 0);
			   }
			else {
				peak_idx[j] = 1;
				d = 1.0 / (1.0 + pch->lr_sca * exp(- pch->lr_exp * pv->diff_snr));
				wt_sum += d;
				gsl_vector_set(peak_wt, j, d);
			}
		}
		
		for (j = 0; j < peak_mat[i].pk_num; j++) {
			pv = &peak_mat[i].pk_vec[j];
			if (peak_idx[j] == 1){
				d = gsl_vector_get(peak_wt, j);
				curr_wt = d / wt_sum;				// current weight of the peak j
				p_sum = 0;
				gsl_vector_set_zero(peak_pp);
				for (k = 0; k < pch->cand_num; k++) {
					fv = pch->cands[k];
					ratio = pv->harm / fv;
					pv->multi = round(ratio);
					pv->resi = ratio - pv->multi;
					rid = (pv->resi + 0.5) / pch->resi_space;
					p_resi = gsl_vector_get(pch->snr_band_resi[pv->abs_sid][pv->bid].pdf_table, rid);
					
					p = pch->snr_band_resi[pv->abs_sid][pv->bid].p_snr * p_resi; // * p_multi;
					p_sum += p;
					gsl_vector_set(peak_pp, k, p);
				}
				for (k = 0; k < pch->cand_num; k++) {
					p = gsl_vector_get(peak_pp, k) / p_sum;
					gsl_vector_set(peak_pp, k, p);
					q = gsl_matrix_get(cand_pp, i, k) + p * curr_wt; //p * pv->diff_snr;
					gsl_matrix_set(cand_pp, i, k, q);
				}
#ifdef SPA_PITCHEXT_DEBUG
				gfp = fopen("debug.txt", "wt");
				gsl_vector_fprintf(gfp, peak_pp, "%e");
				fclose(gfp);
#endif
				p +=p;
			}
		}
		free(peak_idx);
		gsl_vector_free(peak_wt);
		
		// normalization
		p_sum = 0;
		for (k = 0; k < pch->cand_num; k++) {
			p = gsl_matrix_get(cand_pp, i, k);
			p_sum += p;
		}
		for (k = 0; k < pch->cand_num; k++) {
			p = gsl_matrix_get(cand_pp, i, k) / p_sum;
			gsl_matrix_set(cand_pp, i, k, p);
		}
		
#ifdef SPA_PITCHEXT_DEBUG
		gfp = fopen("debug.txt", "wt");
		gsl_vector_view gv = gsl_matrix_row(cand_pp, i);
		gsl_vector_fprintf(gfp, &gv.vector, "%e");
		fclose(gfp);
#endif
	}

	*pcand_pp = cand_pp;
	gsl_vector_free(peak_pp);
}

// generate an lookup table transition cost 
// tab - matrix(cand_num x cand_num), tc_table(i, j) distance from freq i to freq j 
void spa_pitch_get_trans_cost_table(spa_pitch_param *pch)
{
	int i, j;
	int cand_num = pch->cand_num;
	double d, d1, d2;
	double log_two = log(2.0);
	gsl_matrix *tc_table = gsl_matrix_calloc(cand_num, cand_num);
	for (i = 0; i < cand_num; i++) {
		for (j = 0; j < cand_num; j++) {
			d1 = fabs(log(pch->cands[j] / pch->cands[i]));
			d2 = fabs(d1 - log_two) + pch->octave_cost;
			d = d1 < d2 ? d1 : d2;
			gsl_matrix_set(tc_table, i, j, d);
		}
	}
	gsl_matrix_scale(tc_table, pch->dp_ratio * SPA_PITCH_DP_MATCH_SCALE_FACTOR); 	// need to match the scale
	pch->trans_cost_table = tc_table;
}

// using dynamic programing to search a path 
void spa_pitch_cand_dp (spa_pitch_param *pch, int frm_num, gsl_matrix *cand_pp, gsl_vector *uv_ref, gsl_vector **ppch_fin)
{
	int i, j, m, k, l, frm_num_ld, sf, ef, cs, cl;
	int row = frm_num;
	int col = pch->cand_num;
	int *idx;
	double tc, ts, cc, cmax, d, d1;
	gsl_matrix *cost = cand_pp;
	
	gsl_vector *old_acc = gsl_vector_calloc(col);
	gsl_vector *new_acc = gsl_vector_calloc(col);
	gsl_vector *pch_fin = gsl_vector_calloc(row);
	gsl_vector_int *fmpt = gsl_vector_int_calloc(row);		// keep a back pointer of the previous voiced frame 
	gsl_matrix_int *bkpt = gsl_matrix_int_calloc(row, col);		// keep a back pointer of the previous state 
	gsl_vector_set_zero(pch_fin);
	gsl_vector_int_set_zero(fmpt);
	gsl_matrix_int_set_zero(bkpt);
	
	// find the first voiced frame 
	for (i = 0; i < row; i++) {
		if (i < uv_ref->size && gsl_vector_get(uv_ref, i) > 0.5) {
			gsl_matrix_get_row(old_acc, cost, i);
			gsl_vector_int_set(fmpt, i, i);
			break;
		}
	}
	
	// forward i-1 -> i 
	while (i < row - 1) {
		// current frame i is voiced, find the next voiced frame 
		for (j = i + 1; j < row; j++) {
			if (j < uv_ref->size && gsl_vector_get(uv_ref, j) > 0.5) {
				break;
			}
		}		
		if (j >= row) {
			// i is the last voice frame 
			break;
		}
		
		// frame i and j are two consequent frames 
		gsl_vector_int_set(fmpt, j, i); 
		// the time scaling factor is: 
		ts = 1.0 / ( (j - i) * pch->frm_len_sec );
		for (k = 0; k < col; k++) {
			// for all possible states l at time i, find the best one (l*) having the highest accumulative gain to state k at time j*/
			gsl_vector_set(new_acc, k, -DBL_MAX);
			for (l = 0; l < col; l++) {
				tc = gsl_matrix_get(pch->trans_cost_table, l, k);
				d = gsl_vector_get(old_acc, l);
				d1 = tc * ts;
				cc = d - d1;
				if (cc > gsl_vector_get(new_acc, k)) {
					cl = l;
					gsl_vector_set(new_acc, k, cc);	
					gsl_matrix_int_set(bkpt, j, k, l);
				}
			}
			// update accumulative gain at state k by adding the local gain of state k at time j*/
			d = gsl_vector_get(new_acc, k) + gsl_matrix_get(cost, j, k);
			gsl_vector_set(new_acc, k, d);
		}
		// copy new acc to old_acc 
		gsl_vector_memcpy(old_acc, new_acc);
		
		i = j;
		// if i == row - 1 here, loop exit 
	}
	
	// now, the last voiced frame is i, find the max accumulative gain 
	cmax = -DBL_MAX;
	cs = -1;
	for (j = 0; j < col; j++) {
		d = gsl_vector_get(old_acc, j);
		if (d > cmax) {
			cmax = d;
			cs = j;
		}
	}
	// current status is cs, settle the F0 value by back tracking 
	while (1) {
		gsl_vector_set(pch_fin, i, pch->cands[cs]);
		// backward voiced frame i -> the last voiced frame j 
		j = gsl_vector_int_get(fmpt, i);
		if (i == j) {
			// i is the first voiced frame 
			break;
		}
		cs = gsl_matrix_int_get(bkpt, i, cs);
		i = j;
	}
	
	*ppch_fin = pch_fin;
	gsl_vector_free(new_acc);
	gsl_vector_free(old_acc);
	gsl_vector_int_free(fmpt);
	gsl_matrix_int_free(bkpt);
}

